<?php

define('SYS_ROOT_PATH', __dir__ . '/../../../');
include(SYS_ROOT_PATH . 'control/classes/mailer/src/Exception.php');
include(SYS_ROOT_PATH . 'control/classes/mailer/src/SMTP.php');
include(SYS_ROOT_PATH . 'control/classes/mailer/src/PHPMailer.php');

class mailer extends PHPMailer\PHPMailer\PHPMailer {

  private $vars = array();
  public $attachments = array();
  public $smtp_host, $smtp_user, $smtp_pass, $smtp_port;
  
  // Preferred language
  // Must be supported language
  private $mail_lang = 'en';

  // Loads tags into array..
  function addTag($placeholder,$data) {
    $this->vars[$placeholder] = $data;
  }

  // Clears data vars..
  function clearVars() {
    $this->vars = array();
  }

  // Converts tags..
  function convertTags($data) {
    if (!empty($this->vars)) {
      foreach ($this->vars AS $tags => $value) {
        $data = str_replace($tags, $value, $data);
      }
    }

    return $data;
  }

  // Cleans spam/form injection input..
  function safeMail($data) {
    $find = array(
     "\r",
     "\n",
     "%0a",
     "%0d",
     "content-type:",
     "Content-Type:",
     "BCC:",
     "CC:",
     "boundary=",
     "TO:",
     "bcc:",
     "to:",
     "cc:"
     );
    $replace = array();
    return str_replace($find,$replace,$data);
  }

  // Loads plain template..
  function plain($m) {
    return mailer::convertTags(file_get_contents($m['template']));
  }

  // Loads html template..
  function html($m) {
    $f_r = array(
      '{LANG}' => LANG,
      '{DIR}' => DIR,
      '{CHARSET}' => CHARSET,
      '{TITLE}' => $m['html']['header'],
      '{HEADER}' =>$m['html']['header'],
      '{CONTENT}' => nl2br(htmlspecialchars(mailer::convertTags(file_get_contents($m['template']))), false),
      '{FOOTER}' => $m['html']['footer']
    );
    return strtr(file_get_contents($m['html']['wrapper']), $f_r);
  }

  // Sends mail..
  function sendMail($mail = array()) {
    $to_name     = $this->safeMail($mail['to_name']);
    $to_email    = $this->safeMail($mail['to_email']);
    $from_name   = $this->safeMail($mail['from_name']);
    $from_email  = $this->safeMail($mail['from_email']);
    $this->IsSMTP();
    $this->Port = SMTP_PORT;
    $this->Host = SMTP_HOST;
    $this->SMTPAuth = (SMTP_USER && SMTP_PASS ? true : false);
    $this->SMTPSecure = (in_array(SMTP_SECURITY, array(
      '',
      'tls',
      'ssl'
    )) ? SMTP_SECURITY : '');
    if (ALLOW_INSECURE) {
      $this->SMTPOptions = array(
        'ssl' => array(
          'verify_peer' => false,
          'verify_peer_name' => false,
          'allow_self_signed' => true
        )
      );
    }
    $this->Username = SMTP_USER;
    $this->Password = SMTP_PASS;
    $this->From = $from_email;
    $this->FromName = $from_name;
    $this->AddAddress($to_email, $to_name);
    if (isset($mail['add-addresses']) && $mail['add-addresses']) {
      $addEmails = array_map('trim', explode(',', $mail['add-addresses']));
      foreach ($addEmails AS $aAddresses) {
        $this->AddAddress($aAddresses, $mail['to_name']);
      }
    }
    if ($mail['replyto_name'] && $mail['replyto_email']) {
      $this->AddReplyTo($mail['replyto_email'], $mail['replyto_name']);
    }
    if (MAIL_X_MAIL_HEADER) {
      $this->XMailer = MAIL_X_MAIL_HEADER;
    }
    $this->WordWrap = 1000;
    $this->CharSet = MAIL_CHARSET;
    if (SMTP_DEBUG && SMTP_LOG && is_dir(SYS_ROOT_PATH . SMTP_LOG_DIR) && is_writeable(SYS_ROOT_PATH . SMTP_LOG_DIR)) {
      $this->SMTPDebug = 2;
      $this->Debugoutput = function($str, $level) {
        file_put_contents(SYS_ROOT_PATH . SMTP_LOG_DIR . '/' . SMTP_LOG, $str . nl(), FILE_APPEND);
      };
    }
    $this->Subject = $mail['subject'];
    $this->MsgHTML(mailer::html($mail));
    $this->AltBody = mailer::plain($mail);
    if (!empty($this->attachments)) {
      foreach ($this->attachments AS $f => $n) {
        $this->AddAttachment($f, $n);
      }
    }
    // Language
    if ($this->mail_lang != 'en') {
      $this->setLanguage($this->mail_lang, SYS_ROOT_PATH . 'control/classes/mailer/language/');
    }
    $this->Send();
    $this->ClearReplyTos();
    $this->ClearAllRecipients();
    $this->clearAttachments();
  }

}

?>